@@ -1,8 +1,8 @@
-# This is the PyQt5 build script.
+# This is the PyQt6 build script.
 #
-# Copyright (c) 2022 Riverbank Computing Limited <info@riverbankcomputing.com>
+# Copyright (c) 2021 Riverbank Computing Limited <info@riverbankcomputing.com>
 # 
-# This file is part of PyQt5.
+# This file is part of PyQt6.
 # 
 # This file may be used under the terms of the GNU General Public License
 # version 3.0 as published by the Free Software Foundation and appearing in
@@ -27,36 +27,28 @@ from sipbuild import (Buildable, BuildableModule, Installable, Option,
         UserException)
 
 
-# The minimum sip module ABI version needed.
-ABI_VERSION = '12.11'
-
-
 class PyQt(PyQtProject):
-    """ The PyQt5 project. """
+    """ The PyQt6 project. """
 
     def __init__(self):
         """ Initialise the project. """
 
-        # We specify the name of the sip module because PyQt-builder doesn't
-        # provide it if we are creating an sdist.
-        super().__init__(abi_version=ABI_VERSION, sip_module='PyQt5.sip',
-                dunder_init=True, tag_prefix='Qt', console_scripts=[
-                    'pylupdate5 = PyQt5.pylupdate_main:main',
-                    'pyrcc5 = PyQt5.pyrcc_main:main',
-                    'pyuic5 = PyQt5.uic.pyuic:main'])
+        super().__init__(dunder_init=True, tag_prefix='Qt',
+                minimum_glibc_version='2.28', minimum_macos_version='10.14',
+                console_scripts=[
+                    'pylupdate6 = PyQt6.lupdate.pylupdate:main',
+                    'pyuic6 = PyQt6.uic.pyuic:main'])
 
         # Each set of bindings must appear after any set they depend on.
-        self.bindings_factories = [Qt, QtCore, QtNetwork, QtGui, QtWidgets,
-            QtQml, QAxContainer, QtAndroidExtras, QtBluetooth, QtDBus,
-            QtDesigner, Enginio, QtHelp, QtMacExtras, QtMultimedia,
-            QtMultimediaWidgets, QtNfc, QtOpenGL, QtPositioning, QtLocation,
-            QtPrintSupport, QtQuick, QtQuick3D, QtQuickWidgets,
-            QtRemoteObjects, QtSensors, QtSerialPort, QtSql, QtSvg, QtTest,
-            QtTextToSpeech, QtWebChannel, QtWebKit, QtWebKitWidgets,
-            QtWebSockets, QtWinExtras, QtX11Extras, QtXml, QtXmlPatterns,
-            _QOpenGLFunctions_2_0, _QOpenGLFunctions_2_1,
-            _QOpenGLFunctions_4_1_Core, _QOpenGLFunctions_ES2, pylupdate,
-            pyrcc]
+        self.bindings_factories = [QtCore, QtNetwork, QtGui, QtQml, QtWidgets,
+                QtDBus, QtDesigner, QtHelp, QtOpenGL, QtOpenGLWidgets,
+                QtPrintSupport, QtQuick, QtQuick3D, QtQuickWidgets, QtSql,
+                QtSvg, QtSvgWidgets, QtTest, QtXml]
+
+        # These are still to be ported to Qt6:
+        #   QAxContainer, QtBluetooth, QtMultimedia, QtMultimediaWidgets,
+        #   QtNfc, QtPositioning, QtLocation, QtRemoteObjects, QtSensors,
+        #   QtSerialPort, QtTextToSpeech, QtWebChannel, QtWebSockets
 
     def apply_user_defaults(self, tool):
         """ Set default values where needed. """
@@ -83,13 +75,13 @@ class PyQt(PyQtProject):
 def find_qt():
     import os, sys
 
-    qtcore_dll = '\\\\Qt5Core.dll'
+    qtcore_dll = '\\\\Qt6Core.dll'
 
     dll_dir = os.path.dirname(sys.executable)
     if not os.path.isfile(dll_dir + qtcore_dll):
         path = os.environ['PATH']
 
-        dll_dir = os.path.dirname(__file__) + '\\\\Qt5\\\\bin'
+        dll_dir = os.path.dirname(__file__) + '\\\\Qt\\\\bin'
         if os.path.isfile(dll_dir + qtcore_dll):
             path = dll_dir + ';' + path
             os.environ['PATH'] = path
@@ -151,7 +143,7 @@ del find_qt
 
         options.append(
                 Option('tools', option_type=bool, inverted=True,
-                        help="disable the building of pyuic5, pyrcc5 and pylupdate5"))
+                        help="disable the installation of pyuic6 and pylupdate6"))
 
         return options
 
@@ -162,9 +154,9 @@ del find_qt
             return
 
         # Check we support the version of Qt.
-        if self.builder.qt_version >> 16 != 5:
+        if self.builder.qt_version >> 16 != 6:
             raise UserException(
-                    "Qt v5 is required, not v{0}".format(
+                    "Qt v6 is required, not v{0}".format(
                             self.builder.qt_version_str))
 
         # Automatically confirm the license if there might not be a command
@@ -174,15 +166,17 @@ del find_qt
 
         self._check_license()
 
-        # Handle the platform tag (allowing for win32-g++).
-        if self.py_platform.startswith('win32'):
-            plattag = 'WS_WIN'
-        elif self.py_platform == 'darwin':
-            plattag = 'WS_MACX'
-        else:
-            plattag = 'WS_X11'
+        # Handle the platform tag.
+        platform_tags_map = {
+            'android':  'Android',
+            'darwin':   'macOS',
+            'ios':      'iOS',
+            'wasm':     'WebAssembly',
+            'win32':    'Windows',
+        }
 
-        self.bindings['QtCore'].tags.append(plattag)
+        self.bindings['QtCore'].tags.append(
+                platform_tags_map.get(self.py_platform, 'Linux'))
 
         # Make sure the bindings are buildable.
         super().update(tool)
@@ -193,13 +187,14 @@ del find_qt
             if qtcore is not None:
                 qtcore.disabled_features.append('PyQt_WebChannel')
 
-        # Add the composite module.
-        if 'Qt' in self.bindings:
-            self._add_composite_module(tool)
+        # Always install the lupdate module.
+        installable = Installable('lupdate', target_subdir='PyQt6')
+        installable.files.append(os.path.join(self.root_dir, 'lupdate'))
+        self.installables.append(installable)
 
         # Always install the uic module.
-        installable = Installable('uic', target_subdir='PyQt5')
-        installable.files.append(os.path.join(self.root_dir, 'pyuic', 'uic'))
+        installable = Installable('uic', target_subdir='PyQt6')
+        installable.files.append(os.path.join(self.root_dir, 'uic'))
         self.installables.append(installable)
 
         # If any set of bindings is being built as a debug version then assume
@@ -215,37 +210,17 @@ del find_qt
         # This may change when we improve the bundling of Qt.
         if tool in ('build', 'install'):
             if self.designer_plugin and 'QtDesigner' in self.bindings:
-                self._add_plugin('designer', "Qt Designer", 'pyqt5',
+                self._add_plugin('designer', "Qt Designer", 'pyqt6',
                         'designer', others_debug)
 
             if self.qml_plugin and 'QtQml' in self.bindings:
-                self._add_plugin('qmlscene', "qmlscene", 'pyqt5qmlplugin',
-                        'PyQt5', others_debug)
+                self._add_plugin('qmlscene', "qmlscene", 'pyqt6qmlplugin',
+                        'PyQt6', others_debug)
 
         # Add the dbus-python support.
         if self.dbus_python:
             self._add_dbus(others_debug)
 
-    def _add_composite_module(self, tool):
-        """ Add the bindings for the composite module. """
-
-        sip_file = os.path.join(self.build_dir, 'Qt.sip')
-        sip_f = self.open_for_writing(sip_file)
-
-        sip_f.write('''%CompositeModule PyQt5.Qt
-
-''')
-
-        for bindings in self.bindings.values():
-            if not bindings.internal:
-                sip_f.write(
-                        '%Include {}\n'.format(
-                                bindings.sip_file.replace('\\', '/')))
-
-        sip_f.close()
-
-        self.bindings['Qt'].sip_file = sip_file
-
     def _add_dbus(self, debug):
         """ Add the dbus-python support. """
 
@@ -301,7 +276,7 @@ del find_qt
         # Create the buildable.
         sources_dir = os.path.join(self.root_dir, 'dbus')
 
-        buildable = BuildableModule(self, 'dbus', 'dbus.mainloop.pyqt5',
+        buildable = BuildableModule(self, 'dbus', 'dbus.mainloop.pyqt6',
                 uses_limited_api=True)
         buildable.builder_settings.append('QT -= gui')
         buildable.sources.extend(glob.glob(os.path.join(sources_dir, '*.cpp')))
@@ -378,15 +353,14 @@ VPATH = {}
         pro_f.close()
 
     def _check_license(self):
-        """ Handle the validation of the PyQt5 license. """
+        """ Handle the validation of the PyQt6 license. """
 
-        # We read the license.py file as data.
-        license_py = os.path.join(self.root_dir, 'license.py')
+        license_dat = os.path.join(self.root_dir, 'license.dat')
 
-        if os.path.isfile(license_py):
+        if os.path.isfile(license_dat):
             ltype = lname = lfile = None
 
-            with open(license_py) as lf:
+            with open(license_dat) as lf:
                 for line in lf:
                     parts = line.split('=')
                     if len(parts) == 2:
@@ -499,58 +473,6 @@ Type 'no' to decline the terms of the license.
             f5.close()
 
 
-class OpenGLBindings(PyQtBindings):
-    """ Encapsulate internal OpenGL functions bindings. """
-
-    def __init__(self, project, version):
-        """ Initialise the bindings. """
-
-        super().__init__(project, '_QOpenGLFunctions_' + version,
-                test_headers=['qopenglfunctions_{}.h'.format(version.lower())],
-                test_statement='new QOpenGLFunctions_{}()'.format(version),
-                internal=True)
-
-    def is_buildable(self):
-        """ Return True if the bindings are buildable. """
-
-        # Check that the QtGui bindings are being built.
-        qtgui = self.project.bindings.get('QtGui')
-        if qtgui is None:
-            return False
-
-        # Check if all OpenGL support is disabled.
-        if 'PyQt_OpenGL' in qtgui.disabled_features:
-            return False
-
-        # Check if OpenGL desktop support has been disabled.
-        if 'PyQt_Desktop_OpenGL' in qtgui.disabled_features:
-            if self.is_desktop_opengl():
-                return False
-        else:
-            if not self.is_desktop_opengl():
-                return False
-
-        # Run the standard configuration checks.
-        return super().is_buildable()
-
-    def is_desktop_opengl(self):
-        """ Return True if the bindings are for desktop OpenGL. """
-
-        return True
-
-
-class Qt(PyQtBindings):
-    """ The Qt composite module. """
-
-    def __init__(self, project):
-        """ Initialise the bindings. """
-
-        # We specify 'internal' to avoid the generation of .api and .pyi files
-        # and the installation of the .sip files.
-        super().__init__(project, 'Qt', qmake_QT=['-core', '-gui'],
-                internal=True)
-
-
 class QAxContainer(PyQtBindings):
     """ The QAxContainer bindings. """
 
@@ -561,17 +483,6 @@ class QAxContainer(PyQtBindings):
                 test_headers=['qaxobject.h'], test_statement='new QAxObject()')
 
 
-class QtAndroidExtras(PyQtBindings):
-    """ The QtAndroidExtras bindings. """
-
-    def __init__(self, project):
-        """ Initialise the bindings. """
-
-        super().__init__(project, 'QtAndroidExtras',
-                qmake_QT=['androidextras'], test_headers=['QtAndroid'],
-                test_statement='QtAndroid::androidSdkVersion()')
-
-
 class QtBluetooth(PyQtBindings):
     """ The QtBluetooth bindings. """
 
@@ -667,7 +578,8 @@ class QtDesigner(PyQtBindings):
         """ Initialise the bindings. """
 
         super().__init__(project, 'QtDesigner', qmake_QT=['designer'],
-                test_headers=['QExtensionFactory', 'customwidget.h'],
+                test_headers=['QExtensionFactory',
+                        'QtUiPlugin/customwidget.h'],
                 test_statement='new QExtensionFactory()')
 
     def is_buildable(self):
@@ -684,17 +596,6 @@ class QtDesigner(PyQtBindings):
         return super().is_buildable()
 
 
-class Enginio(PyQtBindings):
-    """ The Enginio bindings. """
-
-    def __init__(self, project):
-        """ Initialise the bindings. """
-
-        super().__init__(project, 'Enginio', qmake_QT=['enginio'],
-                test_headers=['enginioclient.h'],
-                test_statement='new EnginioClient()')
-
-
 class QtGui(PyQtBindings):
     """ The QtGui bindings. """
 
@@ -725,17 +626,6 @@ class QtLocation(PyQtBindings):
                 test_headers=['qplace.h'], test_statement='new QPlace()')
 
 
-class QtMacExtras(PyQtBindings):
-    """ The QtMacExtras bindings. """
-
-    def __init__(self, project):
-        """ Initialise the bindings. """
-
-        super().__init__(project, 'QtMacExtras', qmake_QT=['macextras'],
-                test_headers=['qmacpasteboardmime.h'],
-                test_statement='class Foo : public QMacPasteboardMime {}')
-
-
 class QtMultimedia(PyQtBindings):
     """ The QtMultimedia bindings. """
 
@@ -786,7 +676,19 @@ class QtOpenGL(PyQtBindings):
         """ Initialise the bindings. """
 
         super().__init__(project, 'QtOpenGL', qmake_QT=['opengl'],
-                test_headers=['qgl.h'], test_statement='new QGLWidget()')
+                test_headers=['qopengldebug.h'],
+                test_statement='new QOpenGLDebugLogger()')
+
+
+class QtOpenGLWidgets(PyQtBindings):
+    """ The QtOpenGLWidgets bindings. """
+
+    def __init__(self, project):
+        """ Initialise the bindings. """
+
+        super().__init__(project, 'QtOpenGLWidgets',
+                qmake_QT=['openglwidgets'], test_headers=['qopenglwidget.h'],
+                test_statement='new QOpenGLWidget()')
 
 
 class QtPositioning(PyQtBindings):
@@ -904,6 +806,17 @@ class QtSvg(PyQtBindings):
         """ Initialise the bindings. """
 
         super().__init__(project, 'QtSvg', qmake_QT=['svg'],
+                test_headers=['qsvgrenderer.h'],
+                test_statement='new QSvgRenderer()')
+
+
+class QtSvgWidgets(PyQtBindings):
+    """ The QtSvgWidgets bindings. """
+
+    def __init__(self, project):
+        """ Initialise the bindings. """
+
+        super().__init__(project, 'QtSvgWidgets', qmake_QT=['svgwidgets'],
                 test_headers=['qsvgwidget.h'],
                 test_statement='new QSvgWidget()')
 
@@ -942,28 +855,6 @@ class QtWebChannel(PyQtBindings):
                 test_statement='new QWebChannel()')
 
 
-class QtWebKit(PyQtBindings):
-    """ The QtWebKit bindings. """
-
-    def __init__(self, project):
-        """ Initialise the bindings. """
-
-        super().__init__(project, 'QtWebKit', qmake_QT=['webkit', 'network'],
-                test_headers=['qwebkitglobal.h'],
-                test_statement='qWebKitVersion()')
-
-
-class QtWebKitWidgets(PyQtBindings):
-    """ The QtWebKitWidgets bindings. """
-
-    def __init__(self, project):
-        """ Initialise the bindings. """
-
-        super().__init__(project, 'QtWebKitWidgets',
-                qmake_QT=['webkitwidgets', 'printsupport'],
-                test_headers=['qwebpage.h'], test_statement='new QWebPage()')
-
-
 class QtWebSockets(PyQtBindings):
     """ The QtWebSockets bindings. """
 
@@ -985,28 +876,6 @@ class QtWidgets(PyQtBindings):
                 test_headers=['qwidget.h'], test_statement='new QWidget()')
 
 
-class QtWinExtras(PyQtBindings):
-    """ The QtWinExtras bindings. """
-
-    def __init__(self, project):
-        """ Initialise the bindings. """
-
-        super().__init__(project, 'QtWinExtras',
-                qmake_QT=['winextras', 'widgets'], test_headers=['QtWin'],
-                test_statement='QtWin::isCompositionEnabled()')
-
-
-class QtX11Extras(PyQtBindings):
-    """ The QtX11Extras bindings. """
-
-    def __init__(self, project):
-        """ Initialise the bindings. """
-
-        super().__init__(project, 'QtX11Extras', qmake_QT=['x11extras'],
-                test_headers=['QX11Info'],
-                test_statement='QX11Info::display()')
-
-
 class QtXml(PyQtBindings):
     """ The QtXml bindings. """
 
@@ -1015,124 +884,3 @@ class QtXml(PyQtBindings):
 
         super().__init__(project, 'QtXml', qmake_QT=['xml', '-gui'],
                 test_headers=['qdom.h'], test_statement='new QDomDocument()')
-
-
-class QtXmlPatterns(PyQtBindings):
-    """ The QtXmlPatterns bindings. """
-
-    def __init__(self, project):
-        """ Initialise the bindings. """
-
-        super().__init__(project, 'QtXmlPatterns',
-                qmake_QT=['xmlpatterns', '-gui', 'network'],
-                test_headers=['qxmlname.h'], test_statement='new QXmlName()')
-
-
-class _QOpenGLFunctions_2_0(OpenGLBindings):
-    """ The _QOpenGLFunctions_2_0 bindings. """
-
-    def __init__(self, project):
-        """ Initialise the bindings. """
-
-        super().__init__(project, '2_0')
-
-
-class _QOpenGLFunctions_2_1(OpenGLBindings):
-    """ The _QOpenGLFunctions_2_1 bindings. """
-
-    def __init__(self, project):
-        """ Initialise the bindings. """
-
-        super().__init__(project, '2_1')
-
-
-class _QOpenGLFunctions_4_1_Core(OpenGLBindings):
-    """ The _QOpenGLFunctions_4_1_Core bindings. """
-
-    def __init__(self, project):
-        """ Initialise the bindings. """
-
-        super().__init__(project, '4_1_Core')
-
-
-class _QOpenGLFunctions_ES2(OpenGLBindings):
-    """ The _QOpenGLFunctions_ES2 bindings. """
-
-    def __init__(self, project):
-        """ Initialise the bindings. """
-
-        super().__init__(project, 'ES2')
-
-    def is_desktop_opengl(self):
-        """ Return True if the bindings are for desktop OpenGL. """
-
-        return False
-
-
-class pylupdate(PyQtBindings):
-    """ The pylupdate bindings. """
-
-    def __init__(self, project):
-        """ Initialise the bindings. """
-
-        super().__init__(project, 'pylupdate', qmake_QT=['xml', '-gui'],
-                internal=True)
-
-    def generate(self):
-        """ Generate the bindings source code and return the corresponding
-        buildable.
-        """
-
-        # This is re-implemented so that we can update the buildable to include
-        # pylupdate_main.py.
-
-        project = self.project
-
-        buildable = super().generate()
-
-        installable = Installable('pylupdate_main', target_subdir='PyQt5')
-        installable.files.append(
-                os.path.join(project.root_dir, 'pylupdate',
-                        'pylupdate_main.py'))
-        buildable.installables.append(installable)
-
-        return buildable
-
-    def is_buildable(self):
-        """ Return True if the bindings are buildable. """
-
-        return self.project.tools
-
-
-class pyrcc(PyQtBindings):
-    """ The pyrcc bindings. """
-
-    def __init__(self, project):
-        """ Initialise the bindings. """
-
-        super().__init__(project, 'pyrcc', qmake_QT=['xml', '-gui'],
-                internal=True)
-
-    def generate(self):
-        """ Generate the bindings source code and return the corresponding
-        buildable.
-        """
-
-        # This is re-implemented so that we can update the buildable to include
-        # pyrcc_main.py.
-
-        project = self.project
-
-        buildable = super().generate()
-
-        installable = Installable('pyrcc_main', target_subdir='PyQt5')
-        installable.files.append(
-                os.path.join(project.root_dir, 'pyrcc', 'pyrcc_main.py'))
-        buildable.installables.append(installable)
-
-        return buildable
-
-    def is_buildable(self):
-        """ Return True if the bindings are buildable. """
-
-        return self.project.tools
