@@ -1,8 +1,8 @@
 // This is the implementation of the Chimera class.
 //
-// Copyright (c) 2019 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2021 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
-// This file is part of PyQt5.
+// This file is part of PyQt6.
 // 
 // This file may be used under the terms of the GNU General Public License
 // version 3.0 as published by the Free Software Foundation and appearing in
@@ -47,12 +47,89 @@ Chimera::ToQVariantConvertors Chimera::registeredToQVariantConvertors;
 // The registered PyObject to QVariant data convertors.
 Chimera::ToQVariantDataConvertors Chimera::registeredToQVariantDataConvertors;
 
-const int UnknownType = QMetaType::UnknownType;
+
+// The base class for convenience wrappers around mapped type convertors.
+class BaseMappedTypeConvertor
+{
+public:
+    BaseMappedTypeConvertor(const char *name) : td(nullptr), _name(name) {}
+
+protected:
+    void ensureType();
+    const sipTypeDef *td;
+
+private:
+    const char *_name;
+};
+
+
+// Ensure that the type definition is available.
+void BaseMappedTypeConvertor::ensureType()
+{
+    if (!td)
+    {
+        td = sipFindType(_name);
+        Q_ASSERT(td);
+    }
+}
+
+
+// A convenience wrapper around a mapped type convertor.
+template <class T>
+class MappedTypeConvertor : public BaseMappedTypeConvertor
+{
+public:
+    MappedTypeConvertor(const char *name) : BaseMappedTypeConvertor(name) {}
+
+    PyObject *fromMappedType(void *cpp);
+    int toMappedType(PyObject *py, T &cpp);
+};
+
+
+// Convert an instance of the mapped type to a Python object.  A NULL pointer
+// is returned if there was an error.
+template <class T>
+PyObject *MappedTypeConvertor<T>::fromMappedType(void *cpp)
+{
+    ensureType();
+
+    return sipConvertFromNewType(new T(*reinterpret_cast<T *>(cpp)), td, NULL);
+}
+
+
+// Convert a Python object to an instance of the mapped type.  A non-zero value
+// is returned if there was an error.
+template <class T>
+int MappedTypeConvertor<T>::toMappedType(PyObject *py, T &cpp)
+{
+    ensureType();
+
+    int state, iserr = 0;
+    void *p = sipForceConvertToType(py, td, NULL, SIP_NOT_NONE, &state,
+            &iserr);
+
+    if (!iserr)
+    {
+        cpp = *reinterpret_cast<T *>(p);
+        sipReleaseType(p, td, state);
+    }
+
+    return iserr;
+}
+
+
+// The different mapped type convertors.
+static MappedTypeConvertor<QVariantHash> variant_hash(
+        "QHash<QString,QVariant>");
+static MappedTypeConvertor<QVariantList> variant_list("QList<QVariant>");
+static MappedTypeConvertor<QVariantMap> variant_map("QMap<QString,QVariant>");
+static MappedTypeConvertor<QVariantPair> variant_pair(
+        "std::pair<QVariant,QVariant>");
+
 
 // Construct an invalid type.
 Chimera::Chimera()
-    : _type(0), _py_type(0), _metatype(UnknownType), _inexact(false),
-      _is_qflags(false)
+    : _type(0), _py_type(0), _inexact(false), _is_qflags(false)
 {
 }
 
@@ -65,7 +142,7 @@ Chimera::Chimera(const Chimera &other)
     _py_type = other._py_type;
     Py_XINCREF((PyObject *)_py_type);
     
-    _metatype = other._metatype;
+    metatype = other.metatype;
     _inexact = other._inexact;
     _is_qflags = other._is_qflags;
     _name = other._name;
@@ -159,7 +236,7 @@ const Chimera *Chimera::parse(const QMetaProperty &mprop)
     const char *type_name = mprop.typeName();
 
     ct->_type = sipFindType(type_name);
-    ct->_metatype = mprop.userType();
+    ct->metatype = mprop.metaType();
     ct->_inexact = true;
     ct->_is_qflags = mprop.isFlagType();
     ct->_name = type_name;
@@ -370,12 +447,11 @@ bool Chimera::parse_py_type(PyTypeObject *type_obj)
         _type = td;
         _name = sipTypeName(td);
 
-        if (sipTypeIsClass(td))
-            set_qflags();
+        set_qflags(td);
 
         if (sipTypeIsEnum(td) || _is_qflags)
         {
-            _metatype = QMetaType::Int;
+            metatype = QMetaType(QMetaType::Int);
         }
         else
         {
@@ -384,22 +460,22 @@ bool Chimera::parse_py_type(PyTypeObject *type_obj)
             if (!get_assign_helper())
                 _name.append('*');
 
-            _metatype = QMetaType::type(_name.constData());
+            metatype = QMetaType::fromName(_name);
 
             // If it is a user type then it must be a type that SIP knows
             // about but was registered by Qt.
-            if (_metatype < QMetaType::User)
+            if (metatype.id() < QMetaType::User)
             {
                 if (PyType_IsSubtype(type_obj, sipTypeAsPyTypeObject(sipType_QObject)))
                 {
-                    _metatype = QMetaType::QObjectStar;
+                    metatype = QMetaType(QMetaType::QObjectStar);
                 }
                 else if (sipIsUserType((sipWrapperType *)type_obj))
                 {
                     // It is a non-QObject Python sub-class so make sure it
                     // gets wrapped in a PyQt_PyObject.
                     _type = 0;
-                    _metatype = PyQt_PyObject::metatype;
+                    metatype = QMetaType::fromType<PyQt_PyObject>();
                     _name.clear();
                 }
             }
@@ -407,70 +483,49 @@ bool Chimera::parse_py_type(PyTypeObject *type_obj)
     }
     else if (_py_enum_types.contains((PyObject *)type_obj))
     {
-        _metatype = QMetaType::Int;
+        metatype = QMetaType(QMetaType::Int);
         _name = sipPyTypeName(type_obj);
     }
     else if (type_obj == &PyList_Type)
     {
-        _metatype = QMetaType::QVariantList;
+        metatype = QMetaType(QMetaType::QVariantList);
     }
-#if PY_MAJOR_VERSION >= 3
     else if (type_obj == &PyUnicode_Type)
     {
         _type = sipType_QString;
-        _metatype = QMetaType::QString;
+        metatype = QMetaType(QMetaType::QString);
     }
-#else
-    else if (type_obj == &PyString_Type || type_obj == &PyUnicode_Type)
-    {
-        // In this case we accept that the reverse conversion will result in an
-        // object of a different type (i.e. a QString rather than a Python
-        // string).
-        _type = sipType_QString;
-        _metatype = QMetaType::QString;
-    }
-#endif
     else if (type_obj == &PyBool_Type)
     {
-        _metatype = QMetaType::Bool;
+        metatype = QMetaType(QMetaType::Bool);
     }
-#if PY_MAJOR_VERSION < 3
-    else if (type_obj == &PyInt_Type)
-    {
-        // We choose to map to a C++ int, even though a Python int is
-        // potentially much larger, as it represents the most common usage in
-        // Qt.  However we will allow a larger type to be used if the context
-        // is right.
-        _metatype = QMetaType::Int;
-        _inexact = true;
-    }
-#endif
     else if (type_obj == &PyLong_Type)
     {
         // We choose to map to a C++ int for the same reasons as above and to
         // be consistent with Python3 where everything is a long object.  If
         // this isn't appropriate the user can always use a string to specify
         // the exact C++ type they want.
-        _metatype = QMetaType::Int;
+        metatype = QMetaType(QMetaType::Int);
         _inexact = true;
     }
     else if (type_obj == &PyFloat_Type)
     {
-        _metatype = QMetaType::Double;
+        metatype = QMetaType(QMetaType::Double);
     }
     else if (type_obj == sipVoidPtr_Type)
     {
-        _metatype = QMetaType::VoidStar;
+        metatype = QMetaType(QMetaType::VoidStar);
         _name = "void*";
     }
-
-    // Fallback to using a PyQt_PyObject.
-    if (_metatype == UnknownType)
-        _metatype = PyQt_PyObject::metatype;
+    else
+    {
+        // Fallback to using a PyQt_PyObject.
+        metatype = QMetaType::fromType<PyQt_PyObject>();
+    }
 
     // If there is no name so far then use the meta-type name.
     if (_name.isEmpty())
-        _name = QMetaType::typeName(_metatype);
+        _name = metatype.name();
 
     _py_type = type_obj;
     Py_INCREF((PyObject *)_py_type);
@@ -480,10 +535,15 @@ bool Chimera::parse_py_type(PyTypeObject *type_obj)
 
 
 // Set the internal QFlags flag.
-void Chimera::set_qflags()
+void Chimera::set_qflags(const sipTypeDef *td)
 {
-    if (qpycore_is_pyqt_class(_type))
-        _is_qflags = ((pyqt5ClassPluginDef *)sipTypePluginData(_type))->flags & 0x01;
+    if (sipTypeIsMapped(td) && qpycore_is_pyqt_type(_type))
+    {
+        const pyqt6MappedTypePluginDef *plugin = reinterpret_cast<const pyqt6MappedTypePluginDef *>(sipTypePluginData(_type));
+
+        if (plugin)
+            _is_qflags = plugin->flags & 0x01;
+    }
 }
 
 
@@ -616,11 +676,11 @@ bool Chimera::parse_cpp_type(const QByteArray &type)
         return false;
 
     // See if the type is known to Qt.
-    _metatype = QMetaType::type(resolved.constData());
+    metatype = QMetaType::fromName(resolved);
 
     // If not then use the PyQt_PyObject wrapper.
-    if (_metatype == UnknownType)
-        _metatype = PyQt_PyObject::metatype;
+    if (!metatype.isValid())
+        metatype = QMetaType::fromType<PyQt_PyObject>();
 
     // See if the type (without a pointer) is known to SIP.
     bool is_ptr = resolved.endsWith('*');
@@ -645,22 +705,13 @@ bool Chimera::parse_cpp_type(const QByteArray &type)
     if (!_type)
     {
         // This is the only fundamental pointer type recognised by Qt.
-        if (_metatype == QMetaType::VoidStar)
+        if (metatype.id() == QMetaType::VoidStar)
             return true;
 
         // This is 'int', 'bool', etc.
-        if (_metatype != PyQt_PyObject::metatype && !is_ptr)
+        if (metatype != QMetaType::fromType<PyQt_PyObject>() && !is_ptr)
             return true;
 
-        if (resolved == "char" && is_ptr)
-        {
-            // This is a special value meaning a (hopefully) '\0' terminated
-            // string.
-            _metatype = -1;
-
-            return true;
-        }
-
         // This is an explicit 'PyQt_PyObject'.
         if (resolved == "PyQt_PyObject" && !is_ptr)
             return true;
@@ -671,19 +722,14 @@ bool Chimera::parse_cpp_type(const QByteArray &type)
     if (sipTypeIsNamespace(_type))
         return false;
 
-    if (sipTypeIsClass(_type))
+    set_qflags(_type);
+
+    if (sipTypeIsClass(_type) && is_ptr)
     {
-        set_qflags();
+        PyTypeObject *type_obj = sipTypeAsPyTypeObject(_type);
 
-        if (is_ptr)
-        {
-            PyTypeObject *type_obj = sipTypeAsPyTypeObject(_type);
-
-            if (PyType_IsSubtype(type_obj, sipTypeAsPyTypeObject(sipType_QObject)))
-            {
-                _metatype = QMetaType::QObjectStar;
-            }
-        }
+        if (PyType_IsSubtype(type_obj, sipTypeAsPyTypeObject(sipType_QObject)))
+            metatype = QMetaType(QMetaType::QObjectStar);
     }
 
     // We don't support pointers to enums.
@@ -691,7 +737,7 @@ bool Chimera::parse_cpp_type(const QByteArray &type)
         _type = 0;
 
     if (sipTypeIsEnum(_type) || _is_qflags)
-        _metatype = QMetaType::Int;
+        metatype = QMetaType(QMetaType::Int);
 
     return true;
 }
@@ -708,7 +754,7 @@ bool Chimera::fromPyObject(PyObject *py, void *cpp) const
     {
         bool ok;
 
-        if (registeredToQVariantDataConvertors.at(i)(py, cpp, _metatype, &ok))
+        if (registeredToQVariantDataConvertors.at(i)(py, cpp, metatype, &ok))
             return ok;
     }
 
@@ -716,21 +762,22 @@ bool Chimera::fromPyObject(PyObject *py, void *cpp) const
 
     PyErr_Clear();
 
-    switch (_metatype)
+    switch (metatype.id())
     {
+    case QMetaType::Nullptr:
+        *reinterpret_cast<std::nullptr_t *>(cpp) = nullptr;
+        break;
+
     case QMetaType::Bool:
         *reinterpret_cast<bool *>(cpp) = PyLong_AsLong(py);
         break;
 
     case QMetaType::Int:
-        // Truncate it if necessary to fit into a C++ int.  This will
-        // automatically handle enums and flag types as Python knows how to
-        // convert them to ints.
-#if PY_MAJOR_VERSION >= 3
-        *reinterpret_cast<int *>(cpp) = PyLong_AsLong(py);
-#else
-        *reinterpret_cast<int *>(cpp) = PyInt_AsLong(py);
-#endif
+        if (isEnum())
+            *reinterpret_cast<int *>(cpp) = sipConvertToEnum(py, _type);
+        else
+            *reinterpret_cast<int *>(cpp) = PyLong_AsLong(py);
+
         break;
 
     case QMetaType::UInt:
@@ -758,12 +805,36 @@ bool Chimera::fromPyObject(PyObject *py, void *cpp) const
         break;
 
     case QMetaType::Char:
-        if (SIPBytes_Check(py) && SIPBytes_Size(py) == 1)
-            *reinterpret_cast<char *>(cpp) = *SIPBytes_AsString(py);
+        if (PyBytes_Check(py) && PyBytes_Size(py) == 1)
+            *reinterpret_cast<char *>(cpp) = *PyBytes_AsString(py);
         else
             iserr = 1;
         break;
 
+    case QMetaType::Char16:
+        {
+            char16_t ch;
+
+            if (to_char16_t(py, ch))
+                *reinterpret_cast<char16_t *>(cpp) = ch;
+            else
+                iserr = 1;
+
+            break;
+        }
+
+    case QMetaType::Char32:
+        {
+            char32_t ch;
+
+            if (to_char32_t(py, ch))
+                *reinterpret_cast<char32_t *>(cpp) = ch;
+            else
+                iserr = 1;
+
+            break;
+        }
+
     case QMetaType::ULong:
         *reinterpret_cast<unsigned long *>(cpp) = sipLong_AsUnsignedLong(py);
         break;
@@ -777,8 +848,8 @@ bool Chimera::fromPyObject(PyObject *py, void *cpp) const
         break;
 
     case QMetaType::UChar:
-        if (SIPBytes_Check(py) && SIPBytes_Size(py) == 1)
-            *reinterpret_cast<unsigned char *>(cpp) = *SIPBytes_AsString(py);
+        if (PyBytes_Check(py) && PyBytes_Size(py) == 1)
+            *reinterpret_cast<unsigned char *>(cpp) = *PyBytes_AsString(py);
         else
             iserr = 1;
         break;
@@ -792,52 +863,50 @@ bool Chimera::fromPyObject(PyObject *py, void *cpp) const
                 sipType_QObject, 0, SIP_NO_CONVERTORS, 0, &iserr);
         break;
 
+    case QMetaType::QVariantHash:
+        {
+            QVariantHash qvh;
+
+            iserr = variant_hash.toMappedType(py, qvh);
+
+            if (!iserr)
+                *reinterpret_cast<QVariantHash *>(cpp) = qvh;
+
+            break;
+        }
+
     case QMetaType::QVariantList:
         {
-            QVariantList ql;
+            QVariantList qvl;
 
-            if (to_QVariantList(py, ql))
-                *reinterpret_cast<QVariantList *>(cpp) = ql;
-            else
-                iserr = 1;
+            iserr = variant_list.toMappedType(py, qvl);
+
+            if (!iserr)
+                *reinterpret_cast<QVariantList *>(cpp) = qvl;
 
             break;
         }
 
     case QMetaType::QVariantMap:
         {
-            QVariantMap qm;
+            QVariantMap qvm;
 
-            if (qpycore_toQVariantMap(py, qm))
-                *reinterpret_cast<QVariantMap *>(cpp) = qm;
-            else
-                iserr = 1;
+            iserr = variant_map.toMappedType(py, qvm);
+
+            if (!iserr)
+                *reinterpret_cast<QVariantMap *>(cpp) = qvm;
 
             break;
         }
 
-    case QMetaType::QVariantHash:
+    case QMetaType::QVariantPair:
         {
-            QVariantHash qh;
+            QVariantPair qvp;
 
-            if (to_QVariantHash(py, qh))
-                *reinterpret_cast<QVariantHash *>(cpp) = qh;
-            else
-                iserr = 1;
+            iserr = variant_pair.toMappedType(py, qvp);
 
-            break;
-        }
-
-    case -1:
-        {
-            const char **ptr = reinterpret_cast<const char **>(cpp);
-
-            if (SIPBytes_Check(py))
-                *ptr = SIPBytes_AsString(py);
-            else if (py == Py_None)
-                *ptr = 0;
-            else
-                iserr = 1;
+            if (!iserr)
+                *reinterpret_cast<QVariantPair *>(cpp) = qvp;
 
             break;
         }
@@ -914,7 +983,7 @@ bool Chimera::fromPyObject(PyObject *py, QVariant *var, bool strict) const
 {
     // Deal with the simple case of wrapping the Python object rather than
     // converting it.
-    if (_type != sipType_QVariant && _metatype == PyQt_PyObject::metatype)
+    if (_type != sipType_QVariant && metatype == QMetaType::fromType<PyQt_PyObject>())
     {
         // If the type was specified by a Python type (as opposed to
         // 'PyQt_PyObject') then check the object is an instance of it.
@@ -944,6 +1013,7 @@ bool Chimera::fromPyObject(PyObject *py, QVariant *var, bool strict) const
 
     // Temporary storage for different types.
     union {
+        std::nullptr_t tmp_nullptr_t;
         bool tmp_bool;
         int tmp_int;
         unsigned int tmp_unsigned_int;
@@ -953,6 +1023,8 @@ bool Chimera::fromPyObject(PyObject *py, QVariant *var, bool strict) const
         qlonglong tmp_qlonglong;
         short tmp_short;
         char tmp_char;
+        char16_t tmp_char16_t;
+        char32_t tmp_char32_t;
         unsigned long tmp_unsigned_long;
         qulonglong tmp_qulonglong;
         unsigned short tmp_unsigned_short;
@@ -965,23 +1037,27 @@ bool Chimera::fromPyObject(PyObject *py, QVariant *var, bool strict) const
     PyErr_Clear();
 
     QVariant variant;
-    int metatype_used = _metatype;
+    int metatype_used = metatype.id();
 
-    switch (_metatype)
+    switch (metatype.id())
     {
+    case QMetaType::Nullptr:
+        tmp_storage.tmp_nullptr_t = nullptr;
+        break;
+
     case QMetaType::Bool:
         tmp_storage.tmp_bool = PyLong_AsLong(py);
         break;
 
     case QMetaType::Int:
-        if (!_inexact || isEnum())
+        if (isEnum())
+        {
+            tmp_storage.tmp_int = sipConvertToEnum(py, _type);
+        }
+        else if (!_inexact)
         {
             // It must fit into a C++ int.
-#if PY_MAJOR_VERSION >= 3
             tmp_storage.tmp_int = PyLong_AsLong(py);
-#else
-            tmp_storage.tmp_int = PyInt_AsLong(py);
-#endif
         }
         else
         {
@@ -1006,7 +1082,7 @@ bool Chimera::fromPyObject(PyObject *py, QVariant *var, bool strict) const
                     PyErr_Clear();
 
                     *var = keep_as_pyobject(py);
-                    metatype_used = UnknownType;
+                    metatype_used = QMetaType::UnknownType;
                 }
                 else
                 {
@@ -1060,12 +1136,22 @@ bool Chimera::fromPyObject(PyObject *py, QVariant *var, bool strict) const
         break;
 
     case QMetaType::Char:
-        if (SIPBytes_Check(py) && SIPBytes_Size(py) == 1)
-            tmp_storage.tmp_char = *SIPBytes_AsString(py);
+        if (PyBytes_Check(py) && PyBytes_Size(py) == 1)
+            tmp_storage.tmp_char = *PyBytes_AsString(py);
         else
             iserr = 1;
         break;
 
+    case QMetaType::Char16:
+        if (!to_char16_t(py, tmp_storage.tmp_char16_t))
+            iserr = 1;
+        break;
+
+    case QMetaType::Char32:
+        if (!to_char32_t(py, tmp_storage.tmp_char32_t))
+            iserr = 1;
+        break;
+
     case QMetaType::ULong:
         tmp_storage.tmp_unsigned_long = sipLong_AsUnsignedLong(py);
         break;
@@ -1079,8 +1165,8 @@ bool Chimera::fromPyObject(PyObject *py, QVariant *var, bool strict) const
         break;
 
     case QMetaType::UChar:
-        if (SIPBytes_Check(py) && SIPBytes_Size(py) == 1)
-            tmp_storage.tmp_unsigned_char = *SIPBytes_AsString(py);
+        if (PyBytes_Check(py) && PyBytes_Size(py) == 1)
+            tmp_storage.tmp_unsigned_char = *PyBytes_AsString(py);
         else
             iserr = 1;
         break;
@@ -1094,18 +1180,40 @@ bool Chimera::fromPyObject(PyObject *py, QVariant *var, bool strict) const
                 0, SIP_NO_CONVERTORS, 0, &iserr);
         break;
 
+    case QMetaType::QVariantHash:
+        {
+            QVariantHash qvh;
+
+            iserr = variant_hash.toMappedType(py, qvh);
+
+            if (!iserr)
+            {
+                *var = QVariant(qvh);
+                metatype_used = QMetaType::UnknownType;
+            }
+            else if (!strict)
+            {
+                // Assume the failure is because the key was the wrong type.
+                iserr = 0;
+                PyErr_Clear();
+
+                *var = keep_as_pyobject(py);
+                metatype_used = QMetaType::UnknownType;
+            }
+
+            break;
+        }
+
     case QMetaType::QVariantList:
         {
-            QVariantList ql;
+            QVariantList qvl;
 
-            if (to_QVariantList(py, ql))
+            iserr = variant_list.toMappedType(py, qvl);
+
+            if (!iserr)
             {
-                *var = QVariant(ql);
-                metatype_used = UnknownType;
-            }
-            else
-            {
-                iserr = 1;
+                *var = QVariant(qvl);
+                metatype_used = QMetaType::UnknownType;
             }
 
             break;
@@ -1113,58 +1221,43 @@ bool Chimera::fromPyObject(PyObject *py, QVariant *var, bool strict) const
 
     case QMetaType::QVariantMap:
         {
-            QVariantMap qm;
+            QVariantMap qvm;
 
-            if (qpycore_toQVariantMap(py, qm))
+            iserr = variant_map.toMappedType(py, qvm);
+
+            if (!iserr)
             {
-                *var = QVariant(qm);
-                metatype_used = UnknownType;
+                *var = QVariant(qvm);
+                metatype_used = QMetaType::UnknownType;
             }
-            else if (strict)
-            {
-                iserr = 1;
-            }
-            else
+            else if (!strict)
             {
                 // Assume the failure is because the key was the wrong type.
+                iserr = 0;
                 PyErr_Clear();
 
                 *var = keep_as_pyobject(py);
-                metatype_used = UnknownType;
+                metatype_used = QMetaType::UnknownType;
             }
 
             break;
         }
 
-    case QMetaType::QVariantHash:
+    case QMetaType::QVariantPair:
         {
-            QVariantHash qh;
+            QVariantPair qvp;
 
-            if (to_QVariantHash(py, qh))
+            iserr = variant_pair.toMappedType(py, qvp);
+
+            if (!iserr)
             {
-                *var = QVariant(qh);
-                metatype_used = UnknownType;
-            }
-            else
-            {
-                iserr = 1;
+                *var = QVariant::fromValue(qvp);
+                metatype_used = QMetaType::UnknownType;
             }
 
             break;
         }
 
-    case -1:
-        metatype_used = QMetaType::VoidStar;
-
-        if (SIPBytes_Check(py))
-            tmp_storage.tmp_void_ptr = SIPBytes_AsString(py);
-        else if (py == Py_None)
-            tmp_storage.tmp_void_ptr = 0;
-        else
-            iserr = 1;
-
-        break;
-
     default:
         if (_type)
         {
@@ -1202,9 +1295,9 @@ bool Chimera::fromPyObject(PyObject *py, QVariant *var, bool strict) const
     {
         *var = QVariant(*reinterpret_cast<QVariant *>(variant_data));
     }
-    else if (metatype_used != UnknownType)
+    else if (metatype_used != QMetaType::UnknownType)
     {
-        *var = QVariant(metatype_used, variant_data);
+        *var = QVariant(QMetaType(metatype_used), variant_data);
     }
 
     // Release any temporary value-class instance now that QVariant will have
@@ -1244,7 +1337,7 @@ QVariant Chimera::fromAnyPyObject(PyObject *py, int *is_err)
             // If the type is a dict then try and convert it to a QVariantMap
             // if possible.
             if (Py_TYPE(py) == &PyDict_Type)
-                ct._metatype = QMetaType::QVariantMap;
+                ct.metatype = QMetaType(QMetaType::QVariantMap);
 
             // The conversion is non-strict in case the type was a dict and we
             // can't convert it to a QVariantMap.
@@ -1270,7 +1363,7 @@ PyObject *Chimera::toPyObject(const QVariant &var) const
     {
         // For some reason (see qvariant_p.h) an invalid QVariant can be
         // returned when a QMetaType::Void is expected.
-        if (!var.isValid() && _metatype == QMetaType::Void)
+        if (!var.isValid() && metatype.id() == QMetaType::Void)
         {
             Py_INCREF(Py_None);
             return Py_None;
@@ -1278,22 +1371,26 @@ PyObject *Chimera::toPyObject(const QVariant &var) const
 
         // Handle the reverse of non-strict conversions of dict to QVariantMap,
         // ie. we want a dict but we have a QVariantMap.
-        if (_metatype == PyQt_PyObject::metatype && _py_type == &PyDict_Type && var.type() == QVariant::Map)
-            return qpycore_fromQVariantMap(var.toMap());
+        if (metatype == QMetaType::fromType<PyQt_PyObject>() && _py_type == &PyDict_Type && var.metaType().id() == QMetaType::QVariantMap)
+        {
+            QVariantMap qvm = var.toMap();
+
+            return variant_map.fromMappedType(&qvm);
+        }
 
         // A sanity check.
-        if (_metatype != var.userType())
+        if (metatype != var.metaType())
         {
             PyErr_Format(PyExc_TypeError,
                     "unable to convert a QVariant of type %d to a QMetaType of type %d",
-                    var.userType(), _metatype);
+                    var.metaType().id(), metatype.id());
 
             return 0;
         }
 
         // Deal with the simple case of unwrapping a Python object rather than
         // converting it.
-        if (_metatype == PyQt_PyObject::metatype)
+        if (metatype == QMetaType::fromType<PyQt_PyObject>())
         {
             PyQt_PyObject pyobj_wrapper = var.value<PyQt_PyObject>();
 
@@ -1320,14 +1417,14 @@ PyObject *Chimera::toPyObject(const QVariant &var) const
             return py;
     }
 
-    return toPyObject(const_cast<void *>(var.data()));
+    return toPyObject(const_cast<void *>(var.constData()));
 }
 
 
 // Convert a C++ object at an arbitary address to Python.
 PyObject *Chimera::toPyObject(void *cpp) const
 {
-    if (_metatype == PyQt_PyObject::metatype)
+    if (metatype == QMetaType::fromType<PyQt_PyObject>())
     {
         if (_type)
         {
@@ -1359,8 +1456,13 @@ PyObject *Chimera::toPyObject(void *cpp) const
 
     PyObject *py = 0;
 
-    switch (_metatype)
+    switch (metatype.id())
     {
+    case QMetaType::Nullptr:
+        py = Py_None;
+        Py_INCREF(py);
+        break;
+
     case QMetaType::Bool:
         py = PyBool_FromLong(*reinterpret_cast<bool *>(cpp));
         break;
@@ -1376,7 +1478,7 @@ PyObject *Chimera::toPyObject(void *cpp) const
         }
         else
         {
-            py = SIPLong_FromLong(*reinterpret_cast<int *>(cpp));
+            py = PyLong_FromLong(*reinterpret_cast<int *>(cpp));
         }
 
         break;
@@ -1388,7 +1490,7 @@ PyObject *Chimera::toPyObject(void *cpp) const
             if (ui < 0)
                 py = PyLong_FromUnsignedLong((unsigned long)ui);
             else
-                py = SIPLong_FromLong(ui);
+                py = PyLong_FromLong(ui);
 
             break;
         }
@@ -1402,7 +1504,7 @@ PyObject *Chimera::toPyObject(void *cpp) const
         break;
 
     case QMetaType::Long:
-        py = SIPLong_FromLong(*reinterpret_cast<long *>(cpp));
+        py = PyLong_FromLong(*reinterpret_cast<long *>(cpp));
         break;
 
     case QMetaType::LongLong:
@@ -1410,12 +1512,22 @@ PyObject *Chimera::toPyObject(void *cpp) const
         break;
 
     case QMetaType::Short:
-        py = SIPLong_FromLong(*reinterpret_cast<short *>(cpp));
+        py = PyLong_FromLong(*reinterpret_cast<short *>(cpp));
         break;
 
     case QMetaType::Char:
     case QMetaType::UChar:
-        py = SIPBytes_FromStringAndSize(reinterpret_cast<char *>(cpp), 1);
+        py = PyBytes_FromStringAndSize(reinterpret_cast<char *>(cpp), 1);
+        break;
+
+    case QMetaType::Char16:
+        py = PyUnicode_DecodeUTF16(reinterpret_cast<char *>(cpp), 1, NULL,
+                NULL);
+        break;
+
+    case QMetaType::Char32:
+        py = PyUnicode_DecodeUTF32(reinterpret_cast<char *>(cpp), 1, NULL,
+                NULL);
         break;
 
     case QMetaType::ULong:
@@ -1427,7 +1539,7 @@ PyObject *Chimera::toPyObject(void *cpp) const
         break;
 
     case QMetaType::UShort:
-        py = SIPLong_FromLong(*reinterpret_cast<unsigned short *>(cpp));
+        py = PyLong_FromLong(*reinterpret_cast<unsigned short *>(cpp));
         break;
 
     case QMetaType::Float:
@@ -1439,74 +1551,21 @@ PyObject *Chimera::toPyObject(void *cpp) const
                 sipType_QObject, 0);
         break;
 
-    case QMetaType::QVariantList:
-        {
-            QVariantList *ql = reinterpret_cast<QVariantList *>(cpp);
-
-            py = PyList_New(ql->size());
-
-            if (py)
-            {
-                for (int i = 0; i < ql->size(); ++i)
-                {
-                    PyObject *val_obj = toAnyPyObject(ql->at(i));
-
-                    if (!val_obj)
-                    {
-                        Py_DECREF(py);
-                        py = 0;
-
-                        break;
-                    }
-
-                    PyList_SetItem(py, i, val_obj);
-                }
-            }
-
-            break;
-        }
-
-    case QMetaType::QVariantMap:
-        py = qpycore_fromQVariantMap(*reinterpret_cast<QVariantMap *>(cpp));
+    case QMetaType::QVariantHash:
+        py = variant_hash.fromMappedType(cpp);
         break;
 
-    case QMetaType::QVariantHash:
-        {
-            py = PyDict_New();
+    case QMetaType::QVariantList:
+        py = variant_list.fromMappedType(cpp);
+        break;
 
-            if (py)
-            {
-                QVariantHash *qh = reinterpret_cast<QVariantHash *>(cpp);
+    case QMetaType::QVariantMap:
+        py = variant_map.fromMappedType(cpp);
+        break;
 
-                for (QVariantHash::const_iterator it = qh->constBegin(); it != qh->constEnd(); ++it)
-                    if (!addVariantToDict(py, it.key(), it.value()))
-                    {
-                        Py_DECREF(py);
-                        py = 0;
-
-                        break;
-                    }
-            }
-
-            break;
-        }
-
-    case -1:
-        {
-            char *s = *reinterpret_cast<char **>(cpp);
-
-            if (s)
-            {
-                py = SIPBytes_FromString(s);
-            }
-            else
-            {
-                Py_INCREF(Py_None);
-                py = Py_None;
-            }
-
-            break;
-        }
+    case QMetaType::QVariantPair:
+        py = variant_pair.fromMappedType(cpp);
+        break;
 
     default:
         if (_type)
@@ -1523,12 +1582,12 @@ PyObject *Chimera::toPyObject(void *cpp) const
             else
             {
                 // Make a copy as it is a value type.
-                void *copy = QMetaType::create(_metatype, cpp);
+                void *copy = metatype.create(cpp);
 
                 py = sipConvertFromNewType(copy, _type, 0);
 
                 if (!py)
-                    QMetaType::destroy(_metatype, copy);
+                    metatype.destroy(copy);
             }
         }
         else if (_name.contains("_QMLTYPE_"))
@@ -1557,34 +1616,6 @@ PyObject *Chimera::toPyObject(void *cpp) const
 }
 
 
-// Add a QVariant to a Python dict with a QString key.
-bool Chimera::addVariantToDict(PyObject *dict, const QString &key_ref,
-        const QVariant &val_ref)
-{
-    QString *key = new QString(key_ref);
-    PyObject *key_obj = sipConvertFromNewType(key, sipType_QString, 0);
-
-    PyObject *val_obj = toAnyPyObject(val_ref);
-
-    if (!key_obj || !val_obj || PyDict_SetItem(dict, key_obj, val_obj) < 0)
-    {
-        if (key_obj)
-            Py_DECREF(key_obj);
-        else
-            delete key;
-
-        Py_XDECREF(val_obj);
-
-        return false;
-    }
-
-    Py_DECREF(key_obj);
-    Py_DECREF(val_obj);
-
-    return true;
-}
-
-
 // Convert a QVariant to a Python object based on the type of the object.
 PyObject *Chimera::toAnyPyObject(const QVariant &var)
 {
@@ -1610,10 +1641,10 @@ PyObject *Chimera::toAnyPyObject(const QVariant &var)
 
     ct->_type = td;
     ct->_name = type_name;
-    ct->_metatype = var.userType();
+    ct->metatype = var.metaType();
 
-    if (td && sipTypeIsClass(td))
-        ct->set_qflags();
+    if (td)
+        ct->set_qflags(td);
 
     PyObject *py = ct->toPyObject(var);
     delete ct;
@@ -1627,71 +1658,47 @@ QVariant Chimera::keep_as_pyobject(PyObject *py)
 {
     PyQt_PyObject pyobj_wrapper(py);
 
-    return QVariant(PyQt_PyObject::metatype, &pyobj_wrapper);
+    return QVariant(QMetaType::fromType<PyQt_PyObject>(), &pyobj_wrapper);
 }
 
 
-// Convert a Python list object to a QVariantList and return true if there was
-// no error.
-bool Chimera::to_QVariantList(PyObject *py, QVariantList &cpp) const
+// Convert a single character Python string to a char16_t and return true if
+// there was no error.
+bool Chimera::to_char16_t(PyObject *py, char16_t &cpp)
 {
-    Q_ASSERT(PyList_CheckExact(py));
+    if (!PyUnicode_Check(py) || PyUnicode_GetLength(py) != 1)
+        return false;
 
-    for (Py_ssize_t i = 0; i < PyList_Size(py); ++i)
-    {
-        PyObject *val_obj = PyList_GetItem(py, i);
+    PyObject *utf16 = PyUnicode_AsUTF16String(py);
 
-        if (!val_obj)
-            return false;
+    if (!utf16)
+        return false;
 
-        int val_state, iserr = 0;
+    // Skip the BOM.
+    cpp = reinterpret_cast<char16_t *>(PyBytes_AsString(utf16))[1];
 
-        QVariant *val = reinterpret_cast<QVariant *>(sipForceConvertToType(
-                val_obj, sipType_QVariant, 0, SIP_NOT_NONE, &val_state,
-                &iserr));
-
-        if (iserr)
-            return false;
-
-        cpp.append(*val);
-
-        sipReleaseType(val, sipType_QVariant, val_state);
-    }
+    Py_DECREF(utf16);
 
     return true;
 }
 
 
-// Convert a Python object to a QVariantHash and return true if there was no
-// error.
-bool Chimera::to_QVariantHash(PyObject *py, QVariantHash &cpp) const
+// Convert a single character Python string to a char32_t and return true if
+// there was no error.
+bool Chimera::to_char32_t(PyObject *py, char32_t &cpp)
 {
-    Q_ASSERT(PyDict_CheckExact(py));
+    if (!PyUnicode_Check(py) || PyUnicode_GetLength(py) != 1)
+        return false;
 
-    PyObject *key_obj, *val_obj;
-    Py_ssize_t i;
+    PyObject *utf32 = PyUnicode_AsUTF32String(py);
 
-    i = 0;
-    while (PyDict_Next(py, &i, &key_obj, &val_obj))
-    {
-        int key_state, val_state, iserr = 0;
+    if (!utf32)
+        return false;
 
-        QString *key = reinterpret_cast<QString *>(sipForceConvertToType(
-                key_obj, sipType_QString, NULL, SIP_NOT_NONE, &key_state,
-                &iserr));
+    // Skip the BOM.
+    cpp = reinterpret_cast<char32_t *>(PyBytes_AsString(utf32))[1];
 
-        QVariant *val = reinterpret_cast<QVariant *>(sipForceConvertToType(
-                val_obj, sipType_QVariant, NULL, SIP_NOT_NONE, &val_state,
-                &iserr));
-
-        if (iserr)
-            return false;
-
-        cpp.insert(*key, *val);
-
-        sipReleaseType(key, sipType_QString, key_state);
-        sipReleaseType(val, sipType_QVariant, val_state);
-    }
+    Py_DECREF(utf32);
 
     return true;
 }
