@@ -1,8 +1,8 @@
 // This contains the implementation of the pyqtBoundSignal type.
 //
-// Copyright (c) 2020 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2021 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
-// This file is part of PyQt5.
+// This file is part of PyQt6.
 // 
 // This file may be used under the terms of the GNU General Public License
 // version 3.0 as published by the Free Software Foundation and appearing in
@@ -36,13 +36,6 @@
 #include "sipAPIQtCore.h"
 
 
-#if PY_VERSION_HEX >= 0x02050000
-#define CHAR_CAST(s)    (s)
-#else
-#define CHAR_CAST(s)    (const_cast<char *>(s))
-#endif
-
-
 // The type object.
 PyTypeObject *qpycore_pyqtBoundSignal_TypeObject;
 
@@ -52,6 +45,7 @@ extern "C" {
 static PyObject *pyqtBoundSignal_call(PyObject *self, PyObject *args,
         PyObject *kw);
 static void pyqtBoundSignal_dealloc(PyObject *self);
+static Py_hash_t pyqtBoundSignal_hash(PyObject *self);
 static PyObject *pyqtBoundSignal_repr(PyObject *self);
 static PyObject *pyqtBoundSignal_richcompare(PyObject *self, PyObject *other,
         int op);
@@ -112,13 +106,11 @@ PyDoc_STRVAR(pyqtBoundSignal_signal_doc,
 
 // Define the methods.
 static PyMethodDef pyqtBoundSignal_methods[] = {
-    {CHAR_CAST("connect"), (PyCFunction)pyqtBoundSignal_connect,
-            METH_VARARGS|METH_KEYWORDS,
-            CHAR_CAST(pyqtBoundSignal_connect_doc)},
-    {CHAR_CAST("disconnect"), pyqtBoundSignal_disconnect,
-            METH_VARARGS, CHAR_CAST(pyqtBoundSignal_disconnect_doc)},
-    {CHAR_CAST("emit"), pyqtBoundSignal_emit,
-            METH_VARARGS, CHAR_CAST(pyqtBoundSignal_emit_doc)},
+    {"connect", (PyCFunction)pyqtBoundSignal_connect,
+            METH_VARARGS|METH_KEYWORDS, pyqtBoundSignal_connect_doc},
+    {"disconnect", pyqtBoundSignal_disconnect, METH_VARARGS,
+            pyqtBoundSignal_disconnect_doc},
+    {"emit", pyqtBoundSignal_emit, METH_VARARGS, pyqtBoundSignal_emit_doc},
     {0, 0, 0, 0}
 };
 
@@ -132,13 +124,13 @@ static PyGetSetDef pyqtBoundSignal_getset[] = {
 };
 
 
-#if PY_VERSION_HEX >= 0x03040000
 // Define the slots.
 static PyType_Slot qpycore_pyqtBoundSignal_Slots[] = {
     {Py_tp_new,         (void *)PyType_GenericNew},
     {Py_tp_dealloc,     (void *)pyqtBoundSignal_dealloc},
     {Py_tp_repr,        (void *)pyqtBoundSignal_repr},
     {Py_tp_richcompare, (void *)pyqtBoundSignal_richcompare},
+    {Py_tp_hash,        (void *)pyqtBoundSignal_hash},
     {Py_tp_call,        (void *)pyqtBoundSignal_call},
     {Py_mp_subscript,   (void *)pyqtBoundSignal_mp_subscript},
     {Py_tp_methods,     pyqtBoundSignal_methods},
@@ -149,75 +141,12 @@ static PyType_Slot qpycore_pyqtBoundSignal_Slots[] = {
 
 // Define the type.
 static PyType_Spec qpycore_pyqtBoundSignal_Spec = {
-    "PyQt5.QtCore.pyqtBoundSignal",
+    "PyQt6.QtCore.pyqtBoundSignal",
     sizeof (qpycore_pyqtBoundSignal),
     0,
     Py_TPFLAGS_DEFAULT,
     qpycore_pyqtBoundSignal_Slots
 };
-#else
-// Define the mapping methods.
-static PyMappingMethods pyqtBoundSignal_as_mapping = {
-    0,                      /* mp_length */
-    pyqtBoundSignal_mp_subscript,   /* mp_subscript */
-    0,                      /* mp_ass_subscript */
-};
-
-
-// Define the type.
-static PyTypeObject qpycore_pyqtBoundSignal_Type = {
-    PyVarObject_HEAD_INIT(NULL, 0)
-    CHAR_CAST("PyQt5.QtCore.pyqtBoundSignal"),  /* tp_name */
-    sizeof (qpycore_pyqtBoundSignal),   /* tp_basicsize */
-    0,                      /* tp_itemsize */
-    pyqtBoundSignal_dealloc,    /* tp_dealloc */
-    0,                      /* tp_print */
-    0,                      /* tp_getattr */
-    0,                      /* tp_setattr */
-    0,                      /* tp_compare */
-    pyqtBoundSignal_repr,   /* tp_repr */
-    0,                      /* tp_as_number */
-    0,                      /* tp_as_sequence */
-    &pyqtBoundSignal_as_mapping,    /* tp_as_mapping */
-    0,                      /* tp_hash */
-    pyqtBoundSignal_call,   /* tp_call */
-    0,                      /* tp_str */
-    0,                      /* tp_getattro */
-    0,                      /* tp_setattro */
-    0,                      /* tp_as_buffer */
-    Py_TPFLAGS_DEFAULT,     /* tp_flags */
-    0,                      /* tp_doc */
-    0,                      /* tp_traverse */
-    0,                      /* tp_clear */
-    pyqtBoundSignal_richcompare,    /* tp_richcompare */
-    0,                      /* tp_weaklistoffset */
-    0,                      /* tp_iter */
-    0,                      /* tp_iternext */
-    pyqtBoundSignal_methods,    /* tp_methods */
-    0,                      /* tp_members */
-    pyqtBoundSignal_getset,     /* tp_getset */
-    0,                      /* tp_base */
-    0,                      /* tp_dict */
-    0,                      /* tp_descr_get */
-    0,                      /* tp_descr_set */
-    0,                      /* tp_dictoffset */
-    0,                      /* tp_init */
-    0,                      /* tp_alloc */
-    PyType_GenericNew,      /* tp_new */
-    0,                      /* tp_free */
-    0,                      /* tp_is_gc */
-    0,                      /* tp_bases */
-    0,                      /* tp_mro */
-    0,                      /* tp_cache */
-    0,                      /* tp_subclasses */
-    0,                      /* tp_weaklist */
-    0,                      /* tp_del */
-    0,                      /* tp_version_tag */
-#if PY_VERSION_HEX >= 0x03040000
-    0,                      /* tp_finalize */
-#endif
-};
-#endif
 
 
 // The __doc__ getter.
@@ -236,13 +165,7 @@ static PyObject *pyqtBoundSignal_get_doc(PyObject *self, void *)
     if (*docstring == '\1')
         ++docstring;
 
-    return
-#if PY_MAJOR_VERSION >= 3
-        PyUnicode_FromString
-#else
-        PyString_FromString
-#endif
-            (docstring);
+    return PyUnicode_FromString(docstring);
 }
 
 
@@ -251,13 +174,8 @@ static PyObject *pyqtBoundSignal_get_signal(PyObject *self, void *)
 {
     qpycore_pyqtBoundSignal *bs = (qpycore_pyqtBoundSignal *)self;
 
-    return
-#if PY_MAJOR_VERSION >= 3
-        PyUnicode_FromString
-#else
-        PyString_FromString
-#endif
-            (bs->unbound_signal->parsed_signature->signature.constData());
+    return PyUnicode_FromString(
+            bs->unbound_signal->parsed_signature->signature.constData());
 }
 
 
@@ -268,15 +186,9 @@ static PyObject *pyqtBoundSignal_repr(PyObject *self)
 
     QByteArray name = bs->unbound_signal->parsed_signature->name();
 
-    return
-#if PY_MAJOR_VERSION >= 3
-        PyUnicode_FromFormat
-#else
-        PyString_FromFormat
-#endif
-            ("<bound PYQT_SIGNAL %s of %s object at %p>", name.constData() + 1,
-                    sipPyTypeName(Py_TYPE(bs->bound_pyobject)),
-                    bs->bound_pyobject);
+    return PyUnicode_FromFormat("<bound PYQT_SIGNAL %s of %s object at %p>",
+            name.constData() + 1, sipPyTypeName(Py_TYPE(bs->bound_pyobject)),
+            bs->bound_pyobject);
 }
 
 
@@ -315,6 +227,27 @@ static PyObject *pyqtBoundSignal_richcompare(PyObject *self, PyObject *other,
 }
 
 
+// The type hash slot.
+static Py_hash_t pyqtBoundSignal_hash(PyObject *self)
+{
+    qpycore_pyqtBoundSignal *bs = (qpycore_pyqtBoundSignal *)self;
+
+    Py_hash_t signal_hash = PyObject_Hash((PyObject *)(bs->unbound_signal));
+    if (signal_hash == -1)
+        return -1;
+
+    Py_hash_t object_hash = PyObject_Hash((PyObject *)(bs->bound_pyobject));
+    if (object_hash == -1)
+        return -1;
+
+    Py_hash_t hash = signal_hash ^ object_hash;
+    if (hash == -1)
+        hash = -2;
+
+    return hash;
+}
+
+
 // The type call slot.
 static PyObject *pyqtBoundSignal_call(PyObject *self, PyObject *args,
         PyObject *kw)
@@ -340,19 +273,10 @@ static void pyqtBoundSignal_dealloc(PyObject *self)
 // Initialise the type and return true if there was no error.
 bool qpycore_pyqtBoundSignal_init_type()
 {
-#if PY_VERSION_HEX >= 0x03040000
     qpycore_pyqtBoundSignal_TypeObject = (PyTypeObject *)PyType_FromSpec(
             &qpycore_pyqtBoundSignal_Spec);
 
     return qpycore_pyqtBoundSignal_TypeObject;
-#else
-    if (PyType_Ready(&qpycore_pyqtBoundSignal_Type) < 0)
-        return false;
-
-    qpycore_pyqtBoundSignal_TypeObject = &qpycore_pyqtBoundSignal_Type;
-
-    return true;
-#endif
 }
 
 
@@ -410,13 +334,7 @@ static PyObject *pyqtBoundSignal_connect(PyObject *self, PyObject *args,
     PyObject *py_slot, *py_type = 0;
     int no_receiver_check = 0;
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwd_args,
-#if PY_VERSION_HEX >= 0x03030000
-                "O|Op:connect",
-#else
-                "O|Oi:connect",
-#endif
-                const_cast<char **>(kwds), &py_slot, &py_type, &no_receiver_check))
+    if (!PyArg_ParseTupleAndKeywords(args, kwd_args, "O|Op:connect", const_cast<char **>(kwds), &py_slot, &py_type, &no_receiver_check))
         return 0;
 
     Qt::ConnectionType q_type = Qt::AutoConnection;
@@ -871,16 +789,12 @@ static bool get_receiver(PyObject *slot,
         if (!f_name_obj)
             return false;
 
-        PyObject *f_name_owner_obj = f_name_obj;
-        const char *f_name = sipString_AsASCIIString(&f_name_owner_obj);
+        rx_name = qpycore_convert_ASCII(f_name_obj);
         Py_DECREF(f_name_obj);
 
-        if (!f_name)
+        if (rx_name.isNull())
             return false;
 
-        rx_name = f_name;
-        Py_DECREF(f_name_owner_obj);
-
         // See if this has been decorated.
         PyObject *decorations = PyObject_GetAttr(slot_m.pm_function,
                 qpycore_dunder_pyqtsignature);
@@ -1040,7 +954,7 @@ static void slot_signature_from_decorations(QByteArray &slot_signature,
             const Chimera *slot_arg = slot->parsed_arguments.at(a);
 
             // We simply compare meta-types.
-            if (sig_arg->metatype() != slot_arg->metatype())
+            if (sig_arg->metatype != slot_arg->metatype)
             {
                 slot = 0;
                 break;
